require 'json'
require 'net/http'

module Neutrino::Audiomatic::Processor  




  Processors = {
    duration: "/analysis/audio/duration",
    replaygain: "/analysis/audio/replaygain",
    tags: "/analysis/audio/tags",
    webbrowser: "/transcode/audio/webbrowser",
    waveform: "/visualisation/audio/waveform"
  }


  def self.duration(file_url, options = {} ) 
      send_request(processor_url(:duration), options.merge(url: file_url))
  end

  def self.replaygain(file_url, options = {} )
    send_request(processor_url(:replaygain), options.merge(url: file_url))
  end

  def self.tags(file_url, options = {} )
    send_request(processor_url(:tags), options.merge(url: file_url))
  end

  def self.webbrowser(file_url, options = {} )
    send_request(processor_url(:webbrowser), options.merge(url: file_url))
  end

  def self.waveform(file_url, options = {} )
    send_request(processor_url(:waveform), options.merge(url: file_url))
  end

  def self.processor_url(processor)
    URI(Neutrino::Audiomatic.processor_host + Processors[processor])
  end

  def self.send_request( processor_url, body_params = {})
    uri = URI(processor_url)
    user = processor_url.to_s[/https:\/\/(.*):(.*)@/,1]
    password = processor_url.to_s[/https:\/\/(.*):(.*)@/,2]
    https = Net::HTTP.new(uri.host,uri.port)
    https.use_ssl = true

    validation = validate(body_params)
    return validation[:error] unless validation[:valid]
    req = Net::HTTP::Post.new(uri, 'Content-Type' => 'application/json')
    req.basic_auth user, password
    puts format_req_body(body_params)
    req.body =  format_req_body(body_params)
    begin
       res = Net::HTTP.start(processor_url.hostname, processor_url.port) do |http|
       https.request(req) 
     end
   rescue Timeout::Error => e 
    return {error: :unavailble_server, message: "You haven't connection with server"}
  end  

  check_response(res)  
end


private

def self.format_req_body(options)
  JSON({'callback_url' => options[:callback_url], 'url' => options[:url]})
end

def self.check_response res
  case res.code 
  when "401" then 
    {error: :invalid_credentials, message: "Invalid credentials"}
  when "429" then 
    time = res.header['Retry-After']    
    {error: :rate_limit_reached, message: "Rate limit reached. Try in #{time.to_i/60} minutes", meta: time }
  when "202" then
    {success: true, reference: "randomstring"}
  else
    {error: :unknown, message: "Response: code: #{res.code}, body: #{res.body}"}
  end
end 

def self.validate(options = {})
  unless options.include?(:callback_url)
    {valid: false, 
      error: {
        error: :not_configured, 
        message: "Neutrino::Audiomatic hasn't got callback_url"}}           
      else
        {valid: true}
      end
    end
  end  